searchNodes=[{"doc":"A module for changing the power and brightness of the LEDs on a PiGlow device. Each PiGlow device contains 18 LEDs. (See PiGlow.LED for a list.) At any given time, each LED has two hardware properties: enable : Whether power is being supplied to the LED, expressed as 1 / true or 0 / false . power : The amount of power (via PWM ) being supplied to the LED, expressed as an integer between 0 (no power) and 255 (max power). Note that these properties are independent of each other. An LED with enabled set to true may still be turned off if power is set to 0 . Similarly, an LED with power set to 255 will also still be turned off if enable is set to false . In general, the easiest way to use this library is to set all LEDs to enable = true when your application starts, and then simply adjust the power values (using 0 to turn them off). (This is how most other PiGlow libraries work.) However, having access to both properties allows for useful tricks, such as using set_enable/1 to flash the lights on and off — without changing their brightness, and while allowing some lights to remain off ( power = 0 ). To set these properties, three levels of API are available: low: Calling set_* with binary arguments. This will send raw bytes to the device. medium: Calling set_* with a list of values for each LED (in order). This will convert those values to the appropriate binaries, and send them using the &quot;low&quot; method above. high: Calling map_* with a function that maps each PiGlow.LED structure to a value. This will create a list of values, which will be sent via the &quot;medium&quot; method above. Thus, you can choose whichever approach works best for your application — for example, based on efficiency versus complexity. Asynchronous API With the exception of start_link/1 , wait/1 and stop/1 , all functions in this module are asynchronous — they cast messages to the running PiGlow instance without waiting for a reply, and they always return :ok . Aside from generally improving performance, this also allows rapidly queuing up multiple instructions, e.g. iterating through the full brigtness range (from 0 to 255 and back) to &quot;pulse&quot; the LEDs, allowing the PiGlow instance to run at full speed without waits. In situations where you need to ensure that all your prior instructions have completed execution, you can call wait/1 to send a synchronous request to the instance. This will block until the PiGlow instance has finished processing its current message queue. Process lifecycle You generally won't need to start a PiGlow instance manually, as starting the application will (by default) automatically start a named instance, unless you set config :pi_glow, start: false in your application config. If you do choose to manually launch a PiGlow instance, all functions in this module (besides start_link ) accept an optional pid argument that can be used to specify either the PID or registered name of your launched instance. Note that launching or shutting down a PiGlow instance does not change the state of the LEDs . (This is in contrast with other PiGlow libraries, which generally apply power to all LEDs on startup, and may also remove power on exit.) Unless you know the LEDs are already enabled for some other reason, you'll want to use one of the *_enable functions before any *_power changes will be visible. If you are using this library in a script (rather than a daemon), note also that when a script finishes and exits, all in-flight messages are discarded . If your script makes changes to the LEDs and then immediately exits, there's a good chance that most or all of your changes will never get processed. If you want to run LED events just before exiting — for example, to turn them off — be sure to use wait/1 or stop/1 before exiting.","ref":"PiGlow.html","title":"PiGlow","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"PiGlow.html#child_spec/1","title":"PiGlow.child_spec/1","type":"function"},{"doc":"Run a function to determine which LEDs should have power enabled or disabled. The fun argument must be a function that takes one argument (a PiGlow.LED structure) and returns a boolean. The resulting list of booleans will then be sent to set_enable/1 . See that function for more info. Returns :ok immediately. Examples # Turn on the green and blue LEDs, turn off the rest: iex&gt; PiGlow . map_enable ( fn led -&gt; led . colour in [ :green , :blue ] end ) :ok # Turn on five LEDs at random: iex&gt; leds = PiGlow.LED . leds ( ) |&gt; Enum . take_random ( 5 ) iex&gt; PiGlow . map_enable ( &amp; ( &amp;1 in leds ) ) :ok","ref":"PiGlow.html#map_enable/2","title":"PiGlow.map_enable/2","type":"function"},{"doc":"Run a function to determine the enable status and power to send to each LED. The fun argument must be a function that takes one argument (a PiGlow.LED structure) and returns a 2-element tuple {enable, power} , where enable is a boolean and power is an integer in the range of 0..255 . The resulting list of boolean-integer pairs will be unzipped and sent to set_enable_and_power/1 . See that function for more info. Returns :ok immediately. Examples # Enable red LEDs at max brightness, amber at min brightness, disable the rest. iex&gt; PiGlow . map_enable_and_power ( fn ...&gt; %{ colour : :red } -&gt; { true , 255 } ...&gt; %{ colour : :amber } -&gt; { true , 1 } ...&gt; _ -&gt; { false , 0 } ...&gt; end ) :ok","ref":"PiGlow.html#map_enable_and_power/2","title":"PiGlow.map_enable_and_power/2","type":"function"},{"doc":"Run a function to determine how much power to send to each LED. The fun argument must be a function that takes one argument (a PiGlow.LED structure) and returns an integer in the range of 0..255 . The resulting list of integers will then be sent to set_power/1 . See that function for more info. Returns :ok immediately. Examples # Set all LEDs to random brightness: iex&gt; PiGlow . map_power ( fn _ -&gt; Enum . random ( 1 .. 255 ) end ) :ok # Set the first arm to max brightness, the second to medium, the third to minimum. iex&gt; PiGlow . map_power ( fn ...&gt; %{ arm : 1 } -&gt; 255 ...&gt; %{ arm : 2 } -&gt; 125 ...&gt; %{ arm : 3 } -&gt; 1 ...&gt; end ) :ok","ref":"PiGlow.html#map_power/2","title":"PiGlow.map_power/2","type":"function"},{"doc":"Enables or disables power to each LED. The values argument can be specified one of two ways: As a list of 18 booleans, with true or false indicating whether each LED should be enabled or disabled. As a 3-byte binary, where each byte is a 6-bit integer indicating which LEDs should be enabled. Note that LEDs require both enabled = true and power &gt; 0 to light up. Use set_power/1 to adjust LED power, or set_enabled_and_power/1 to set both in a single operation. Always returns :ok immediately (even if there is no PiGlow process running). Use wait/1 if you need to ensure your changes have been sent to the device. Examples # Enable LEDs at indices 1, 2, 3, 5, 8, 13: iex&gt; 1 .. 18 |&gt; Enum . map ( &amp; ( &amp;1 in [ 1 , 2 , 3 , 5 , 8 , 13 ] ) ) |&gt; PiGlow . set_enable ( ) :ok # Equivalent to: iex&gt; PiGlow . set_enable ( &lt;&lt; 0b111010 , 0b010000 , 0b100000 &gt;&gt; ) :ok","ref":"PiGlow.html#set_enable/2","title":"PiGlow.set_enable/2","type":"function"},{"doc":"Enables or disables power to all LEDs, and sets the amount of power, in a single operation. The values argument can be specified one of two ways: As a list of 18 two-element tuples, each in the format {enable, power} , where enable is a boolean and power is an integer. As a two-element tuple, in the format {enable_values, power_values} . enable_values can be in either format (binary or list) accepted by set_enable/1 . power_values can be in either format (binary or list) accepted by set_power/1 . When using this function, both instructions are sent to the I2C controller, one immediately after the other, with no &quot;update&quot; operation sent inbetween. This should avoid any unexpected LED flickering caused by setting each value independently. Always returns :ok immediately (even if there is no PiGlow process running). Use wait/1 if you need to ensure your changes have been sent to the device. Examples # Set all LEDs to minimum brightness: iex&gt; 1 .. 18 |&gt; Enum . map ( fn _ -&gt; 1 end ) |&gt; PiGlow . set_power ( ) :ok # Equivalent to: iex&gt; PiGlow . set_power ( &lt;&lt; 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 &gt;&gt; ) :ok","ref":"PiGlow.html#set_enable_and_power/2","title":"PiGlow.set_enable_and_power/2","type":"function"},{"doc":"Sets the amount of power being delivered to each LED. The values argument can be specified one of two ways: As a list of 18 integers, ranging from 0 (off) to 255 (full power). As an 18-byte binary, where each byte is an integer, as above. Note that LEDs require both enabled = true and power &gt; 0 to light up. Use set_enable/1 to turn LEDs on, or set_enabled_and_power/1 to set both in a single operation. Always returns :ok immediately (even if there is no PiGlow process running). Use wait/1 if you need to ensure your changes have been sent to the device. Examples # Set all LEDs to minimum brightness: iex&gt; 1 .. 18 |&gt; Enum . map ( fn _ -&gt; 1 end ) |&gt; PiGlow . set_power ( ) :ok # Equivalent to: iex&gt; PiGlow . set_power ( &lt;&lt; 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 , 1 &gt;&gt; ) :ok","ref":"PiGlow.html#set_power/2","title":"PiGlow.set_power/2","type":"function"},{"doc":"Starts a process that will update the PiGlow device LEDs when it receives messages. Options :bus - I2C bus device number (default: 1 ) :name - Registered process name to use (default: PiGlow ) Use value nil to prevent registration. This function also accepts all the options accepted by GenServer.start_link/3 . Return values Same as GenServer.start_link/3 .","ref":"PiGlow.html#start_link/1","title":"PiGlow.start_link/1","type":"function"},{"doc":"Stops a running instance and releases the I2C device. Returns :ok once all pending messages are processed and the instance has been cleanly stopped. Note that PiGlow uses a default restart policy of :transient , meaning that it will not be automatically restarted if stopped via this function. If no message is received within timeout milliseconds (default: 60 seconds), the caller will exit. Examples # Turn off all LEDs, then shut it down: iex&gt; PiGlow . map_enable_and_power ( fn _ -&gt; { false , 0 } end ) :ok iex&gt; PiGlow . stop ( ) :ok","ref":"PiGlow.html#stop/2","title":"PiGlow.stop/2","type":"function"},{"doc":"Waits for all prior messages to be received and processed. Returns :ok once all pending messages are processed. (Note that this does not guarantee that the PiGlow is idle, only that messages sent prior to the start of this wait call have been processed.) If no message is received within timeout milliseconds (default: 60 seconds), the caller will exit, as per standard GenServer.call/2 semantics. Examples # Pulse all lights once: iex&gt; [ 0 .. 255 , 255 .. 0 ] |&gt; ...&gt; Enum . flat_map ( &amp; Enum . to_list / 1 ) |&gt; ...&gt; Enum . map ( &amp; PiGlow.LED . gamma_correct / 1 ) |&gt; ...&gt; Enum . each ( fn value -&gt; ...&gt; PiGlow . map_power ( fn _ -&gt; value end ) ...&gt; end ) :ok # Wait for those 512 events to all be processed: iex&gt; PiGlow . wait ( ) :ok","ref":"PiGlow.html#wait/2","title":"PiGlow.wait/2","type":"function"},{"doc":"","ref":"PiGlow.html#t:enable/0","title":"PiGlow.enable/0","type":"type"},{"doc":"","ref":"PiGlow.html#t:power/0","title":"PiGlow.power/0","type":"type"},{"doc":"A structure representing a single LED on a PiGlow device. These are the values that the PiGlow.map_* functions pass to the user-supplied function. Your application code can choose which LEDs to operate on by looking at the LED's properties: index — the position of the LED in the binary versions of PiGlow.set_enable and PiGlow.set_power arm — which of the three arms the LED is on (clockwise) 1 — the top arm, directly below the &quot;PiGlow&quot; text 2 — the right arm, on the side with the Pimoroni URL 3 — the left arm, on the side with the small text and icons ring — how far from the centre the LED is, in the range of 1..6 colour — the LED colour, as an atom Note that ring and colour are effectively the same value, just expressed two different ways: ring 1 corresponds to colour :white ring 2 corresponds to colour :blue ring 3 corresponds to colour :green ring 4 corresponds to colour :amber ring 5 corresponds to colour :orange ring 6 corresponds to colour :red Additionally, this module also contains utility functions relating to LEDs and LED brightness.","ref":"PiGlow.LED.html","title":"PiGlow.LED","type":"module"},{"doc":"Calculates the power value needed to approximate a given brightness. The relationship between how much power is sent to an LED, and how bright that LED actually shines, is a non-linear one — i.e. going from 90% to 95% of max brightness requires a significantly larger increase in energy than going from 5% to 10% of max brightness. This function attempts to correct for that by applying an exponential curve to the supplied brightness value. The value argument can be either an integer between 0 and 255 inclusive, or a float between 0.0 and 1.0 inclusive. The result will follow this pattern: gamma_correct(v) = 0 when v is 0 or 0.0 gamma_correct(v) = 255 when v is 255 or 1.0 otherwise, gamma_correct(v) will follow an exponential curve from 1 to 255 Returns an integer in the range of 0..255 , which can be passed to the power -based functions in PiGlow . Examples # Pulse all lights once: iex&gt; [ 0 .. 255 , 255 .. 0 ] |&gt; ...&gt; Enum . flat_map ( &amp; Enum . to_list / 1 ) |&gt; ...&gt; Enum . map ( &amp; PiGlow.LED . gamma_correct / 1 ) |&gt; ...&gt; Enum . each ( fn value -&gt; ...&gt; PiGlow . map_power ( fn _ -&gt; value end ) ...&gt; end ) :ok","ref":"PiGlow.LED.html#gamma_correct/1","title":"PiGlow.LED.gamma_correct/1","type":"function"},{"doc":"Return the list of LEDs on a PiGlow device. Examples # Find all green LEDs: iex&gt; PiGlow.LED . leds ( ) |&gt; Enum . filter ( fn led -&gt; led . colour == :green end ) [ % PiGlow.LED { index : 6 , arm : 1 , ring : 3 , colour : :green } , % PiGlow.LED { index : 14 , arm : 2 , ring : 3 , colour : :green } , % PiGlow.LED { index : 4 , arm : 3 , ring : 3 , colour : :green } ]","ref":"PiGlow.LED.html#leds/0","title":"PiGlow.LED.leds/0","type":"function"},{"doc":"","ref":"PiGlow.LED.html#t:colour/0","title":"PiGlow.LED.colour/0","type":"type"},{"doc":"","ref":"PiGlow.LED.html#t:t/0","title":"PiGlow.LED.t/0","type":"type"},{"doc":"PiGlow is a library for controlling a Pimoroni &quot;PiGlow&quot; LED array . With it, you can turn the LEDs on or off, and adjust their power output, allowing your Elixir daemon to give some fun visual feedback to the world around it.","ref":"readme.html","title":"PiGlow","type":"extras"},{"doc":"PiGlow requires Elixir v1.14. To use it, add :pi_glow to your list of dependencies in mix.exs : def deps do [ { :pi_glow , &quot;~&gt; 0.1.0&quot; } ] end","ref":"readme.html#installation","title":"PiGlow - Installation","type":"extras"},{"doc":"# Set all LEDs to full power, but turned off: PiGlow . map_enable_and_power ( fn _ -&gt; { false , 255 } end ) # Turn on each arm in sequence: 1 .. 3 |&gt; Enum . each ( fn arm -&gt; PiGlow . map_enable ( fn led -&gt; led . arm == arm end ) Process . sleep ( 1000 ) end ) # Turn everything off: PiGlow . map_enable_and_power ( fn _ -&gt; { false , 0 } end ) # Wait before exiting: PiGlow . wait ( ) More examples can be found in the examples directory.","ref":"readme.html#usage","title":"PiGlow - Usage","type":"extras"},{"doc":"Full documentation can be found at https://hexdocs.pm/pi_glow .","ref":"readme.html#documentation","title":"PiGlow - Documentation","type":"extras"},{"doc":"Copyright © 2023, Adrian Irving-Beer. PiGlow is released under the MIT license and is provided with no warranty . I doubt it's possible to damage your PiGlow with this library, but if you do, I'm not responsible.","ref":"readme.html#legal-stuff","title":"PiGlow - Legal stuff","type":"extras"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Changelog created. Initial release.","ref":"changelog.html#v0-1-0","title":"Changelog - v0.1.0","type":"extras"}]